""" 
Freedom server application

@copyright: 2011 Nir Soffer <nirsof@gmail.com>, Erez Waisbard <waisbard@gmail.com>
@license: GNU GPL, see COPYING for details.
"""

import sys
import logging
import getpass
import time
import optparse
from email import charset

from freedom import imap
from freedom import xmpp
from freedom import smtp
from freedom import page
from freedom import config
from freedom import event

XMPP_HOST = ('talk.google.com', 5222)
IMAP_HOST = 'imap.gmail.com'
SMTP_HOST = 'smtp.gmail.com'

def main(args):
    parser = make_parser()
    options, args = parser.parse_args(args, config)
    configure_logger(options)
    if not args:
        parser.error('email-address required')
    user = args[0]
    password = getpass.getpass('Password: ')
    if options.quoted_printable:
        logging.info('using quoted printable encoding')
        # Change the strange default to encode utf-8 body using base64. It
        # woule be nicer to modify only a single message, but it does not
        # work. The only way to do this seems to be changing the global
        # mappping.  See
        # http://radix.twistedmatrix.com/2010/07/how-to-send-good-unicode-email-with.html
        charset.add_charset('utf-8', charset.QP, charset.QP, 'utf-8')
    if options.event_log:
        event.init(options.event_log)
    responder = smtp.responder(SMTP_HOST, user, password)
    responder.start()
    downloader = page.downloader(responder)
    downloader.start()
    imap_listener = imap.request_listener(IMAP_HOST, user, password, downloader) 
    imap_listener.start()
    xmpp_listener = xmpp.listener(XMPP_HOST, user, password, downloader)
    xmpp_listener.start()
    try:
        while 1:
            time.sleep(60)
    except KeyboardInterrupt:
        pass
    if options.event_log:
        event.shutdown()
    logging.shutdown()

def make_parser():
    parser = optparse.OptionParser(usage='%prog [options] email-address')
    parser.add_option('-d', '--debug', dest='debug', action='store_true',
            help='show debug log message and keep temporary page directories (default %s).' % config.debug)
    parser.add_option('--quoted-printable', dest='quoted_printable', type='string',
            metavar='BOOL', action='callback', callback=store_bool,
            help='encode embedded page text using quoted printable encoding (default %s).' % config.quoted_printable)
    parser.add_option('--event-log', dest='event_log',
            help='enable event logging mode, using given file. You must run the client and the server on the same machine and use the same log file in both.')
    return parser

def store_bool(option, opt_str, value, parser):
    setattr(parser.values, option.dest, boolify(value)) 
    
def boolify(value):
    return value.lower() in ('1', 'true', 'yes')

def configure_logger(options):
    if options.debug:
        level = logging.DEBUG
    else:
        level = logging.INFO
    logging.basicConfig(level=level,
            format='[%(asctime)s] [%(levelname)s] %(message)s')

if __name__ ==  '__main__':
    args = sys.argv[1:]
    main(args)

