"""
Freedom SMTP responder

The SMTP responder send responses to the client.

@copyright: 2011 Nir Soffer <nirsof@gmail.com>, Erez Waisbard <waisbard@gmail.com>
@license: GNU GPL, see COPYING for details.
"""

import email
import smtplib
import logging
import time

from freedom import dispatch
from freedom import event

class responder(object):

    def __init__(self, host, user, password, debug=False):
        self._host = host
        self._user = user
        self._password = password
        self._debug = debug
        self._queue = dispatch.queue(self)
        self._conn = None
    
    ## Running

    def start(self):
        self._queue.start()

    def stop(self):
        self._queue.stop()

    ## Handling requests

    def handle(self, request):
        self._queue.handle(request)

    ## dispatch.queue delegate methods

    def queue_handle(self, request):
        event.log('server.connect', time.time())
        self._connect()
        event.log('server.send', time.time())
        self._send_response(request)
        event.log('server.finished', time.time())
        self._disconnect()

    def _connect(self):
        logging.info('smtp.responder connecting to %r', self._host)
        start = time.time()
        self._conn = smtplib.SMTP(self._host, 587)
        self._conn.set_debuglevel(self._debug)
        self._conn.ehlo()
        logging.debug('smtp.responder starting tls')
        self._conn.starttls()
        self._conn.ehlo()
        logging.info('smtp.responder logging in as %r', self._user)
        self._conn.login(self._user, self._password)
        logging.debug('clock: connect: %.2fs', time.time() - start)
    
    def _send_response(self, request):
        start = time.time()
        msg = request['response']
        self._prepare_response(msg, request) 
        data = msg.as_string()
        logging.info('smtp.responder sending response to: %r url: %r filename: %r length: %d', 
                request['from'], request['url'], request['filename'], len(data))
        self._conn.sendmail(self._user, request['from'], data)
        logging.debug('clock: send %.2fs', time.time() - start)

    def _prepare_response(self, msg, request):
        msg['from'] = self._user
        msg['to'] = request['from']
        msg['subject'] = 'Re: ' + request['url']
        msg['x-freedom-url'] = request['url']

    def _disconnect(self):
        self._conn.quit()
        self._conn = None

