#!/usr/bin/env python2.7
""" Freedom event log statistics

Run this on the log file created by client and server running in event logging
mode (--event-log).

@copyright: 2011 Nir Soffer <nirsof@gmail.com>, Erez Waisbard <waisbard@gmail.com>
@license: GNU GPL, see COPYING for details.
"""

import sys
import json
import math

XMPP, DOWNLOAD, BUILD, CONNECT, SEND, GMAIL, RECEIVE, TOTAL = range(8)

columns = ['xmpp', 'downlaod', 'build', 'connect', 'send', 'gmail', 'receive', 'total']

def meandev(p, i):
    n = float(len(p))
    mean = math.fsum(row[i] for row in p) / n
    variance = math.fsum((row[i] - mean)**2 for row in p) / n
    stddev = math.sqrt(variance)
    return mean, stddev

def print_table(title, p):
    head = '  %10s  %8s  %6s%%  %8s' % ('part', 'mean', '', 'stddev')
    print
    print '%s: %d' % (title, len(p))
    print head
    print '-' * len(head)

    total_mean, total_stddev = meandev(p, TOTAL)

    for i, col in enumerate(columns[:-1]):
        mean, stddev = meandev(p, i)
        prec = mean / total_mean * 100
        print '  %10s  %8.3f  %6.1f%% %8.3f' % (col, mean, prec, stddev)
    
    print '  %10s  %8.3f  %6.1f%% %8.3f' % ('total', total_mean, 100, total_stddev)


for filename in sys.argv[1:]:

    requests = []
    common = []
    noncommon = []

    for line in open(filename):
        try:
            ev = json.loads(line)
        except ValueError, e:
            print 'invalid record: %r' % line.strip()
            continue
        t = dict(ev)
        total = t["client.open"] - t["tester.send"]
        xmpp = t['server.download'] - t['tester.send']
        download = t['server.build'] - t['server.download'] 
        build = t['server.connect'] - t['server.build']
        connect = t['server.send'] - t['server.connect']
        send = t['server.finished'] - t['server.send']
        gmail = t['client.receive'] - t['server.finished']
        receive = t['client.open'] - t['client.receive']
        requests.append((xmpp, download, build, connect, send, gmail, receive, total))

    print
    print filename
    print '=' * len(filename)
    print
    print 'total requests: %d' % len(requests)
    
    # Separte requests differing more the the standard deviation
    mean, stddev = meandev(requests, TOTAL)
    for row in requests:
        if abs(row[TOTAL] - mean) > 2 * stddev:
            noncommon.append(row)
        else:
            common.append(row)

    print_table('Common requests', common)
    if noncommon:
        print_table('Non common requests', noncommon)

